<?php

namespace Drupal\cfrpreset\Configurator;

use Drupal\cfrapi\CfrCodegenHelper\CfrCodegenHelperInterface;
use Drupal\cfrapi\Configurator\ConfiguratorInterface;
use Drupal\cfrapi\Configurator\Unconfigurable\Configurator_OptionlessBase;
use Drupal\cfrapi\SummaryBuilder\SummaryBuilderInterface;

class Configurator_CfrPreset extends Configurator_OptionlessBase {

  /**
   * @var string
   */
  private $interface;

  /**
   * @var string
   */
  private $presetMachineName;

  /**
   * @var \Drupal\cfrapi\Configurator\ConfiguratorInterface
   */
  private $typeConfigurator;

  /**
   * @var mixed
   */
  private $conf;

  /**
   * @var bool
   */
  private $initialized = FALSE;

  /**
   * @var string|null
   */
  private $php;

  /**
   * @var mixed|null
   */
  private $value;

  /**
   * @param string $interface
   * @param string $presetMachineName
   * @param mixed $conf
   *
   * @return \Drupal\cfrpreset\Configurator\Configurator_CfrPreset
   */
  public static function create($interface, $presetMachineName, $conf) {
    return new self(
      $interface,
      $presetMachineName,
      cfrplugin()->interfaceGetConfigurator($interface),
      $conf);
  }

  /**
   * @param string $interface
   * @param string $presetMachineName
   * @param \Drupal\cfrapi\Configurator\ConfiguratorInterface $typeConfigurator
   * @param mixed $conf
   */
  public function __construct($interface, $presetMachineName, ConfiguratorInterface $typeConfigurator, $conf) {
    $this->interface = $interface;
    $this->presetMachineName = $presetMachineName;
    $this->typeConfigurator = $typeConfigurator;
    $this->conf = $conf;
  }

  /**
   * @param mixed $conf_ignore
   *   Configuration from a form, config file or storage.
   * @param string|null $label
   *   Label for the form element, specifying the purpose where it is used.
   *
   * @return array
   */
  public function confGetForm($conf_ignore, $label) {

    if (!user_access('administer cfrpreset')) {
      return [];
    }

    $preset = _cfrpreset_load_preset($this->interface, $this->presetMachineName);
    $link = l(
      t('edit preset'),
      _cfrpreset_preset_get_path($preset),
      ['attributes' => ['target' => '_blank']]);

    return [
      '#type' => 'container',
      'link' => [
        '#markup' => $link,
      ],
    ];
  }

  /**
   * @param mixed $conf_ignore
   *   Configuration from a form, config file or storage.
   * @param \Drupal\cfrapi\SummaryBuilder\SummaryBuilderInterface $summaryBuilder
   *   An object that controls the format of the summary.
   *
   * @return mixed|string|null
   *   A string summary is always allowed. But other values may be returned if
   *   $summaryBuilder generates them.
   */
  public function confGetSummary($conf_ignore, SummaryBuilderInterface $summaryBuilder) {

    return NULL;
  }

  /**
   * @param mixed $conf_ignore
   *   Configuration from a form, config file or storage.
   *
   * @return mixed
   *   Value to be used in the application.
   */
  public function confGetValue($conf_ignore) {
    // Run this only once.
    // @todo Remember even in case of exception?
    return $this->initialized
      ? $this->value
      : $this->value = $this->createValue();
  }

  /**
   * @param mixed $conf
   *   Configuration from a form, config file or storage.
   * @param \Drupal\cfrapi\CfrCodegenHelper\CfrCodegenHelperInterface $helper
   *
   * @return string
   *   PHP statement to generate the value.
   */
  public function confGetPhp($conf, CfrCodegenHelperInterface $helper) {

    return (NULL !== $this->php)
      ? $this->php
      : $this->php = $this->generatePhp($helper);
  }

  /**
   * @return mixed
   *
   * @throws \Drupal\cfrapi\Exception\InvalidConfigurationException
   */
  private function createValue() {
    static $recursion_level = 0;
    if ($recursion_level > 12) {
      // @todo Return error value?
      return NULL;
    }
    $current_recursion_level = $recursion_level;
    ++$recursion_level;

    try {
      return $this->typeConfigurator->confGetValue($this->conf);
    }
    // Let the exception slip through.
    finally {
      $recursion_level = $current_recursion_level;
    }
  }

  /**
   * @param \Drupal\cfrapi\CfrCodegenHelper\CfrCodegenHelperInterface $helper
   *
   * @return string
   */
  private function generatePhp(CfrCodegenHelperInterface $helper) {

    static $recursion_level = 0;
    if ($recursion_level > 12) {
      return $helper->recursionDetected(NULL, 'Recursion in cfrpreset.');
    }
    $current_recursion_level = $recursion_level;
    ++$recursion_level;

    $php = $this->typeConfigurator->confGetPhp($this->conf, $helper);

    $recursion_level = $current_recursion_level;

    return '// Preset: ' . str_replace("\n", "\n// ", $this->presetMachineName)
      . "\n" . $php;
  }
}
